# Summary

- [Introduction](./introduction.md)
- [Tutorial](./tutorial.md)
  - [Creating `hello-world.wasm`](./tutorial-create-hello-world.md)
  - [Running `hello-world.wasm`](./tutorial-run-hello-world.md)
- [Examples](./examples.md)
  - [Markdown parser](./examples-markdown.md)
  - [Profiling WebAssembly](./examples-profiling.md)
    - [Profiling with Perf](./examples-profiling-perf.md)
    - [Profiling with VTune](./examples-profiling-vtune.md)
  - [Embedding in Rust](./examples-rust-embed.md)
    - [Hello, world!](./examples-rust-hello-world.md)
    - [Calculating the GCD](./examples-rust-gcd.md)
    - [Using linear memory](./examples-rust-memory.md)
    - [WASI](./examples-rust-wasi.md)
    - [Linking modules](./examples-rust-linking.md)
    - [Debugging](./examples-rust-debugging.md)
    - [Using multi-value](./examples-rust-multi-value.md)
  - [Embedding in C](./examples-c-embed.md)
    - [Hello, world!](./examples-c-hello-world.md)
    - [Calculating the GCD](./examples-c-gcd.md)
    - [Using linear memory](./examples-c-memory.md)
    - [WASI](./examples-c-wasi.md)
    - [Linking modules](./examples-c-linking.md)
    - [Debugging](./examples-c-debugging.md)
    - [Using multi-value](./examples-c-multi-value.md)
- [Using WebAssembly from your lanugage](./lang.md)
  - [Rust](./lang-rust.md)
  - [C](./lang-c.md)
  - [Python](./lang-python.md)
  - [.NET](./lang-dotnet.md)
  - [Go](./lang-go.md)
  - [Bash](./lang-bash.md)
- [Using the `wasmtime` CLI](./cli.md)
  - [Installation](./cli-install.md)
  - [CLI Options](./cli-options.md)
  - [Cache Configuration](./cli-cache.md)
- [Writing WebAssembly](./wasm.md)
  - [Rust](./wasm-rust.md)
  - [C/C++](./wasm-c.md)
  - [AssemblyScript](./wasm-assemblyscript.md)
  - [WebAssembly Text Format (`*.wat`)](./wasm-wat.md)
  - [Example: Markdown Parser](./wasm-markdown.md)
- [Stability](stability.md)
  - [Release Process](./stability-release.md)
  - [Platform Support](./stability-platform-support.md)
- [Security](security.md)
  - [Disclosure Policy](./security-disclosure.md)
  - [Sandboxing](./security-sandboxing.md)
- [Contributing](contributing.md)
  - [Building](./contributing-building.md)
  - [Testing](./contributing-testing.md)
  - [Fuzzing](./contributing-fuzzing.md)
  - [CI](./contributing-ci.md)
  - [Coding guidelines](./contributing-coding-guidelines.md)
  - [Development process](./contributing-development-process.md)
  - [Release Process](./contributing-release-process.md)
  - [Governance](./contributing-governance.md)
  - [Code of Conduct](./contributing-coc.md)
